/*
 Name:		DiodenTester.ino
 Created:	27.12.2022
 Author:	   Ulli
*/

#include "Voltage.h"
#include "Adc.h"

int test(); // Fhrt den Test durch

const uint8_t pin1 = A0;
const AdcChannel channel1 = AdcChannel::ADC0; // Pin A0

const AdcChannel channel2 = AdcChannel::ADC1; // Pin A1

const uint8_t pin3 = A2;
const AdcChannel channel3 = AdcChannel::ADC2; // Pin A2

const uint8_t ledTx = 1; // TX-Pin
const uint8_t ledBi = LED_BUILTIN; // eingebaute LED

bool toggle = false; // zum Blinken
Voltage USi(700);    // Durchlassspannung Si-Diode

void setup() {
   pinMode(pin1, OUTPUT);
   pinMode(pin3, OUTPUT);
   pinMode(ledBi, OUTPUT);
   pinMode(ledTx, OUTPUT);
}

void loop() {
   delay(200);  // Bestimmt die Blink-Frequenz
   toggle = !toggle;

   int result = test();
   if (result == -2) { // Kurzschluss, beide LEDs an
      digitalWrite(ledBi, HIGH);
      digitalWrite(ledTx, LOW);
      return;
   }

   if (result == -1) { // Keine Diode, beide LEDs aus
      digitalWrite(ledBi, LOW);
      digitalWrite(ledTx, HIGH);
      return;
   }

   auto UF = Voltage(result); // Umwandeln in Spannung zum Vergleichen

   if (UF == USi) { // Si-Diode, LEDs blinken synchron
      digitalWrite(ledBi, toggle);  // LED gegen Masse
      digitalWrite(ledTx, !toggle); // LED gegen Vcc
   }
   else {  // andere Diode, LEDs blinken abwechselnd
      digitalWrite(ledBi, toggle); // LED gegen Masse
      digitalWrite(ledTx, toggle); // LED gegen Vcc
   }
}

// Fhrt den Test durch. Liefert:
// -1: Keine Diode
// -2: Kurzschluss
// >0: UF
int test() {
   uint16_t vcc = Adc.readVcc();
   Adc.setReference(AdcReference::Vcc, vcc);

   // Schritt 1:
   digitalWrite(pin1, HIGH);
   digitalWrite(pin3, LOW);
   auto U2 = Adc.readAvgmv(channel2, 16);
   auto U3 = Adc.readAvgmv(channel3, 16);
   if (U2 == U3) { // Kurzschluss
      return -2;
   }

   // Schritt 2:
   auto U1 = Adc.readAvgmv(channel1, 16);

   if (U2 < U1) { // leitende Diode
      return U2 - U3;
   }

   // Schritt 3:
   // Jetzt kann noch sperrende oder keine Diode angeschlossen sein
   // Polung umkehren
   digitalWrite(pin1, LOW);
   digitalWrite(pin3, HIGH);
   U1 = Adc.readAvgmv(channel1, 16);
   U2 = Adc.readAvgmv(channel2, 16);
   U3 = Adc.readAvgmv(channel3, 16);

   if (U2 == U1) {
      return -1;
   }

   // Schritt 4:
   return U3 - U2;
}